import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ExtensibleURIConverterImpl;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

/**
 * Solution using absolute URIs to create resources. EMF takes care about the
 * schemaLocation automatically.
 */
public class GoodSolution {
	
	public static void main(String[] args) {
		// Create ResourceSet and register factories
		ResourceSet rs = new ResourceSetImpl();
		Map<String, Object> map = rs.getResourceFactoryRegistry().getExtensionToFactoryMap();
		map.put("ecore", new EcoreResourceFactoryImpl());
		map.put("xmi", new XMIResourceFactoryImpl());
		
		// Load the meta model dynamically and register it in the package registry of the ResourceSet 
		URI ecoreUri = URI.createFileURI(new File("input/metaModel.ecore").getAbsolutePath());
		Resource rMeta = rs.getResource(ecoreUri, true);
		EPackage metamodel = (EPackage)rMeta.getContents().get(0);
		rs.getPackageRegistry().put(metamodel.getNsURI(), metamodel);	
		
		// Load the input instance model dynamically
		URI inputUri = URI.createFileURI(new File("input/instanceModel.xmi").getAbsolutePath());
		Resource rInput = rs.getResource(inputUri, true);
		EObject inputRoot = rInput.getContents().get(0);
		
		// ... Optimisation would take place here ...
		
		// Create an output resource for the optimised model instance and serialize it
		URI outputUri = URI.createFileURI(new File("results/run1/offspring.xmi").getAbsolutePath());
		Resource rOut = rs.createResource(outputUri);
		rOut.getContents().clear();
		rOut.getContents().add(inputRoot);

		Map<Object,Object> options = new HashMap<Object,Object>();		
		options.put(XMIResource.OPTION_SCHEMA_LOCATION, Boolean.TRUE);
		
		try {
			rOut.save(options);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
