import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ExtensibleURIConverterImpl;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

/**
 * This solution solves the problem by defining the output directory as the base bath
 * used by the URIConverter. That way, the relative URI needed to load the meta model
 * is also the correct URI to be used as the schemaLocation in the output model.
 * 
 * While it works for this situation, this solution is generally not good. You will get
 * into trouble as soon as there are multiple output paths (e.g. "results/run2"). 
 * Additionally, I am not sure how exactly the use of relative paths will affect the
 * resolution of cross references. To stay on the safe side, use absolute URIs to create
 * resources (see GoodSolution.java). In that case, EMF will care about the correct 
 * schemaLocations.  
 */
public class WorkingButBadSolution {
	
	public static void main(String[] args) {
		// Create ResourceSet and register factories
		ResourceSet rs = new ResourceSetImpl();
		Map<String, Object> map = rs.getResourceFactoryRegistry().getExtensionToFactoryMap();
		map.put("ecore", new EcoreResourceFactoryImpl());
		map.put("xmi", new XMIResourceFactoryImpl());
		
		// Define a base path and allow the ResourceSet to resolve relative URIs against this base path
		String basePath = new File("results/run1").getAbsolutePath() + File.separatorChar;
		URI baseUri = URI.createFileURI(basePath);	
		rs.setURIConverter(new ExtensibleURIConverterImpl() {
			@Override
			public URI normalize(URI uri) {
				if (uri.isFile() && uri.isRelative() && baseUri!=null) {
					return uri.resolve(baseUri);
				} else {
					return super.normalize(uri);
				}
			}	
		});		
		
		// Load the meta model dynamically and register it in the package registry of the ResourceSet 
		URI ecoreUri = URI.createFileURI("../../input/metaModel.ecore");
		Resource rMeta = rs.getResource(ecoreUri, true);
		EPackage metamodel = (EPackage)rMeta.getContents().get(0);
		rs.getPackageRegistry().put(metamodel.getNsURI(), metamodel);	
			
		// Load the input instance model dynamically
		URI inputUri = URI.createFileURI("../../input/instanceModel.xmi");
		Resource rInput = rs.getResource(inputUri, true);
		EObject inputRoot = rInput.getContents().get(0);
		
		// ... Optimisation would take place here ...
		
		// Create an output resource for the optimised model instance and serialize it
		URI outputUri = URI.createFileURI("optimisedModel.xmi");
		Resource rOut = rs.createResource(outputUri);
		rOut.getContents().clear();
		rOut.getContents().add(inputRoot);

		Map<Object,Object> options = new HashMap<Object,Object>();		
		options.put(XMIResource.OPTION_SCHEMA_LOCATION, Boolean.TRUE);
		
		try {
			rOut.save(options);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
